/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEMOPLUGIN_H
#define DEMOPLUGIN_H

#include <stdint.h>

class DemoPlugin {
public:
    DemoPlugin(const char *filename);
    ~DemoPlugin();
    void load();

    /* Initialize the plugin with the given resolution and possible
     configuration file name (can be NULL). */
    void init(int width, int height, const char *config_file);
    /* Render a frame to the given preallocated texture buffer. Format
     is 32 bpp RGBA, size defined by init(). The time given should be
     used in preference to measuring time yourself. */
    void render(uint32_t time_in_milliseconds);
    /* Run the garbage collector of the plugin, if it has one. */
    void gc(void);
    /* Try to clean up after the plugin. */
    void shutdown(void);

    unsigned char* getTexture() { return texture; }

    int getTextureWidth() const { return w; }
    int getTextureHeight() const { return h; }

private:
    const char *filename;
    void *library_handle;
    int w, h;

    unsigned char *texture;

    void *init_function;
    void *render_function;
    void *gc_function;
    void *shutdown_function;

    void clearTexture();
};

#endif
