/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "demoplugin.h"
#include "demolib.h"
#include <assert.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <string.h>
#include <stdio.h>

DemoPlugin::DemoPlugin(const char *filename) {
    this->filename = filename;
    library_handle = NULL;
    texture = NULL;
    w = h = -1;
}

DemoPlugin::~DemoPlugin() {
    if (library_handle)
        dlclose(library_handle);
    library_handle = NULL;
    delete []texture;
    texture = NULL;
}

void DemoPlugin::load() {
    assert(!library_handle);
    int flags = RTLD_LAZY;
    if (!strcmp("./libpythonplugin.so", filename)) {
        log_print(debugLevel, "Activating Python kludge!\n");
        flags |= RTLD_GLOBAL;
    }
        
    library_handle = dlopen(filename, flags);
    if (!library_handle)
        throw dlerror();
#define GET_FUNC(name) name##_function = dlsym(library_handle, #name); assert(name##_function);
    GET_FUNC(init);
    GET_FUNC(render);
    GET_FUNC(gc);
    GET_FUNC(shutdown);
#undef GET_FUNC
}

void DemoPlugin::init(int width, int height, const char *config_file) {
    assert(library_handle);
    ((void (*)(int, int, const char *))init_function)(width, height, config_file);
    delete [] texture;
    w = width;
    h = height;
    texture = new unsigned char[w*h*4];
    clearTexture();
}

void DemoPlugin::render(uint32_t time_in_milliseconds) {
    assert(library_handle);
    assert(texture != NULL);
    ((void (*)(unsigned char *, uint32_t))render_function)(texture, time_in_milliseconds);
}

void DemoPlugin::gc(void) {
    assert(library_handle);
    ((void (*)(void))gc_function)();
}

void DemoPlugin::shutdown(void) {
    assert(library_handle);
    ((void (*)(void))shutdown_function)();
}

void DemoPlugin::clearTexture() {
    for (int i=0; i<w; i++) {
        for (int j=0; j<h; j++) {
            unsigned char *base = texture + (j*w + i)*4;
            base[0] = 0;
            base[1] = 0;
            base[2] = 0;
            base[3] = 255;
        }
    }
}
