/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef DEMORUNNER_H
#define DEMORUNNER_H

#include <tr1/unordered_map>
#include <vector>
#include <iostream>
#include <fstream>
#include "SDL_opengl.h"
#include "demoplugin.h"
#include "pluginthread.h"
#include "SDL.h"
#include "videomanager.h"
#include "scenerunner.h"
#include "audiomanager.h"

enum overlayType {
    staticType,
    linearType,
    smoothType,
    fadeIn,
    fadeOut
};

struct basicOverlay {
    std::string texName;
    overlayType type;
    double x, y, w, h;
    int startTime, endTime;
    double x1, y1;
};

struct basicScene {
    std::string name;
    int duration;
    std::vector<std::string> plugins;
    std::vector<basicOverlay> overlays;
};

struct textureSize {
    int w;
    int h;
};

class UserExit : std::exception {};

class DemoRunner {

    // This class needs several pieces of data from us, so it is easier to just set it to friend.
    friend class SceneRunner;

private:
    std::vector<PluginThread*> pluginThreads;
    std::tr1::unordered_map<std::string, PluginThread*> pluginThreadNameMap;
    std::tr1::unordered_map<std::string, GLuint> textureMap;
    std::tr1::unordered_map<std::string, textureSize> textureSizeMap;
    VideoManager v;
    Uint32 zeroTime; // The "epoch" for all timing operations.
    SceneRunner scene;
    AudioManager audio;

    void loadTexture(const std::string &texname, const char *fname, std::string texfilter);
    GLuint surfaceToTexture(SDL_Surface *im, std::string texfilter);
    void updatePluginTexture(GLuint texid, PluginThread *pt);
    void finalizeGraphics();
    std::string readFirstNonCommentToken(std::ifstream &iFile);
    void parseSceneFile(const char *sceneFile, std::vector<basicScene> &scenes);
    void parseSingleScene(std::ifstream &inFile, basicScene &s);
    void drawOverlays(std::vector<basicOverlay> &overlays, Uint32);
    void runScenes(std::vector<basicScene> &scenes);
    void parseLogLevel(std::string &level);

public:

    DemoRunner(int screenW, int screenH, bool fullScreen);
    void initGraphics();
    void parseInit(const char *initFile);
    void parseTextures(const char *initFile);
    void shutdown();
    void parseFonts(const char *initFile);
    PluginThread* getPluginThread(const char *pluginName) { return pluginThreadNameMap[pluginName]; }
    GLuint getGLTexId(const char *rscName) { return textureMap[rscName]; }
    Uint32 getTime(); // This is the master clock.

    void renderTest();
    void runDemo(const char *sceneFile);
};

#endif
