/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SDL.h"
#include "SDL_opengl.h"
#include "SDL_image.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include"demoplugin.h"
#include"config.h"

const int WIDTH = 800;
const int HEIGHT = 600;
int texh = 256;
int texw = 256;

SDL_Surface *init_graphics(int w, int h, bool fullscreen) {
    Uint32 surfaceFlags = SDL_OPENGL;
    SDL_Surface *s;
    int depth = 32;

    if(fullscreen)
        surfaceFlags |= SDL_FULLSCREEN;

    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    s = SDL_SetVideoMode(w, h, depth, surfaceFlags);
    return s;
}

void opengl_setup() {
    glShadeModel(GL_SMOOTH);
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glColor3d(1.0, 1.0, 1.0);
    glClearDepth(1.0f);
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void switchTo2DMode() {
    glDisable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(-1.0, 1.0, -1.0, 1.0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void switchTo3DMode() {
    glEnable(GL_DEPTH_TEST);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f,(GLfloat)WIDTH/(GLfloat)HEIGHT,0.1f,100.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void draw_overlay(double dx, double dy) {
    glPushMatrix();
    glTranslated(dx, dy, 0);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f, -0.5f,  0.0f);
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.5f, -0.5f,  0.0f);
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 0.5f,  0.5f,  0.0f);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f,  0.5f,  0.0f);
    glEnd();
    glPopMatrix();
    glFinish();
}

void draw3DStuff(double angle) {
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glTranslated(0.0, 0.0, -2.0);
    glRotated(angle, 0.0, 1.0, 0.0);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f, -0.5f,  0.0f);
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 0.5f, -0.5f,  0.0f);
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 0.5f,  0.5f,  0.0f);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f,  0.5f,  0.0f);
    glEnd();
    glPopMatrix();
}

void viewTest(const char *image) {
    SDL_Event e;
    SDL_Surface *tex = IMG_Load(image);
    GLuint texid[1];
    int moveItMoveIt = 0;

    glGenTextures(1, &texid[0]);
    glBindTexture(GL_TEXTURE_2D, texid[0]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexImage2D(GL_TEXTURE_2D, 0, 4, tex->w, tex->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, tex->pixels);

    SDL_FreeSurface(tex);

    while(1) {
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        switchTo3DMode();
        draw3DStuff(moveItMoveIt*3.6);
        draw3DStuff(moveItMoveIt*3.6 + 90.0);
        switchTo2DMode();
        draw_overlay(moveItMoveIt/100.0, moveItMoveIt/100.0);
        SDL_GL_SwapBuffers();

        SDL_Delay(100);
        moveItMoveIt = (++moveItMoveIt) % 100;
        // Check if we need to exit.
        while(SDL_PollEvent(&e)) {
            if (e.type == SDL_QUIT ||
                (e.type == SDL_KEYDOWN &&
                (e.key.keysym.sym == SDLK_ESCAPE || e.key.keysym.sym == SDLK_q))) {
                glDeleteTextures(1, &texid[0]);
                return;
            }
        }
    }
}

int main(int argc, char **argv) {
    const int isFullscreen = false;
    SDL_Surface *canvas;
    const char *conffile;

    printf("OpenGL 2D drawing test\n");

    if(SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0) {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        return 1;
    }
    SDL_WM_SetCaption("GL2D", "GL2D");

    int sdlimageFlags = IMG_INIT_JPG|IMG_INIT_PNG;
    int sdlimageInit = IMG_Init(sdlimageFlags);
    if(sdlimageInit & sdlimageInit != sdlimageFlags) {
        printf("IMG_Init failed: %s\n", IMG_GetError());
        return 1;
    }

    canvas = init_graphics(WIDTH, HEIGHT, isFullscreen);
    if(!canvas) {
        fprintf(stderr, "Could not initialize video: %s\n", SDL_GetError());
        exit(1);
    }

    opengl_setup();
    try {
        viewTest("../data/testimage.png");
    } catch (char *ex) {
        printf("Got exception: %s\n", ex);
        return 1;
    }

    IMG_Quit();
    SDL_Quit();
    return 0;
}
