/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include"SDL.h"
#include"SDL_opengl.h"
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<string>

#include"config.h"
#include"demorunner.h"
#include"demolib.h"

const int WIDTH  = 1024;
const int HEIGHT = 768;

void printVideoModes() {
    SDL_Rect** modes;
    int i;

    modes = SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_HWSURFACE);

    if (modes == (SDL_Rect**) 0) {
        log_print(debugLevel, "No modes available!\n");
        exit(-1);
    }

    if (modes == (SDL_Rect**) -1) {
        log_print(debugLevel, "All resolutions available.\n");
    } else {
        log_print(debugLevel, "Available fullscreen Modes:\n");
        for (i = 0; modes[i]; ++i)
            log_print(debugLevel, "  %d x %d\n", modes[i]->w, modes[i]->h);
    }
}

int main(int argc, char **argv) {
    const int isFullscreen = true;
    DemoRunner dr(WIDTH, HEIGHT, isFullscreen);
    SDL_Surface *canvas;

    log_print(messageLevel, "%s\n", FULL_ID_STRING);

    SDL_WM_SetCaption(PROGRAM_NAME, PROGRAM_NAME);

    try {
        dr.initGraphics();
        log_print(debugLevel, "Graphics initialised.\n");
        dr.parseInit("../data/initialize.txt");
        log_print(debugLevel, "Plugin initialisation done.\n");
        printVideoModes();
        dr.parseTextures("../data/texturedefinitions.txt");
        log_print(debugLevel, "Parsed textures.\n");
        dr.parseFonts("../data/fontdefinitions.txt");
        log_print(debugLevel, "Parsed fonts.\n");
        if (argc > 1) {
            dr.runDemo(argv[1]);
        } else {
            dr.renderTest();
        }
        dr.shutdown();
    } catch (const char *e) {
        log_print(errorLevel, "Got exception: %s\n", e);
    } catch (char *e) {
        log_print(errorLevel, "Got exception: %s\n", e);
    } catch (std::string e) {
        log_print(errorLevel, "Got exception: %s\n", e.c_str());
    }

    SDL_Quit();
    return 0;
}
