/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <SDL.h>

#include "demoplugin.h"

using namespace std;

#define MILLIS_PER_FRAME (1000/60)
#define DIRECTORY_MODE 0755

void write_image_rgba_raw(char *texture, const char *output_filename,
        int width, int height) {
    ofstream output_file(output_filename);
    output_file.write(texture, width * height * 4);
    output_file.close();
}

void write_image_rgba_bmp(unsigned char *texture, const char *output_filename,
        int width, int height) {
    const Uint32 red_mask = 0xff;
    const Uint32 green_mask = 0xff00;
    const Uint32 blue_mask = 0xff0000;
    const Uint32 alpha_mask = 0xff000000;

    SDL_Surface *surface = SDL_CreateRGBSurfaceFrom(texture, width, height, 32,
            width * 4, red_mask, green_mask, blue_mask, alpha_mask);
    if (-1 == SDL_SaveBMP(surface, output_filename)) {
        cerr << "Error saving frame." << endl;
        abort();
    }
    SDL_FreeSurface(surface);
}

int main(int argc, char **argv) {
    char *plugin_name = argv[1];
    char *config_file = NULL;
    int width = 256, height = 256;
    int n_frames = 1;
    const char *output_dir = "output";

    if (argc < 2 || argc > 6) {
        fprintf(
                stderr,
                "Usage: %s <plugin.so> [<config file>] [<width> <height>] [<n_frames>]\n",
                argv[0]);
        exit(1);
    }

    if (argc >= 3) {
        config_file = argv[2];
        printf("Using %s as config file for plugin.\n", config_file);
    }

    if (argc >= 5) {
        width = atoi(argv[3]);
        height = atoi(argv[4]);
    }

    if (argc == 6)
        n_frames = atoi(argv[5]);

    SDL_Init(SDL_INIT_VIDEO);

    printf("Size: %d x %d\n", width, height);
    try {
        DemoPlugin plugin(plugin_name);

        cout << "Loading plugin " << plugin_name << endl;
        plugin.load();

        cout << "Calling init function..." << endl;
        plugin.init(width, height, config_file);

        mkdir(output_dir, DIRECTORY_MODE);
        for (int frame = 1; frame <= n_frames; frame++) {
            cout << "Calling render function for frame " << frame << endl;
            plugin.render((frame - 1) * MILLIS_PER_FRAME);

            char output_filename[256];
            sprintf(output_filename, "%s/frame%04d.bmp", output_dir, frame);
            cout << "Writing image in BMP format to " << output_filename
                    << endl;
            write_image_rgba_bmp(plugin.getTexture(), output_filename, width,
                    height);
        }

        cout << "Calling gc..." << endl;
        plugin.gc();

        cout << "Calling shutdown..." << endl;
        plugin.shutdown();
    } catch (char *message) {
        cerr << "Error: " << message << endl;
    }
    return 0;
}

