/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "SDL.h"
#include "SDL_image.h"
#include <assert.h>
#include <stdlib.h>

#include "pluginapi.h"

const int w=16;
const int h=16;

SDL_Surface *backImage;
const unsigned char *originalPalette;

void init(int width, int height, const char *config_file) {
    assert(width == w);
    assert(height == h);
    backImage = IMG_Load(config_file);
    assert(backImage->pitch == backImage->w*3);
    originalPalette = backImage->pixels;
}

void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    unsigned char *dst = texture;
    const unsigned char *src;
    int i, shift;
    char cmd[1024];
    int shifts[256];
    FILE *f;

    shift = (time_in_milliseconds/100) % 16;

    snprintf(cmd, 1024, "./adaeffect %d > palette.txt", shift);
    i = system(cmd);
    assert(i==0);
    f = fopen("palette.txt", "r");
    for (i=0; i<w*h; i++) {
        fscanf(f, "%d", &shift);
        shifts[i] = shift;
    }
    fclose(f);

    // Now assign new palette entries and write to texture.
    for (i=0; i<w*h; i++) {
        src = originalPalette + 3*shifts[i];
        dst[0] = src[0];
        dst[1] = src[1];
        dst[2] = src[2];
        dst += 4;
    }
}

void gc(void) {

}

void shutdown(void) {
    SDL_FreeSurface(backImage);
}


