/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<stdlib.h>
#include<stdio.h>
#include<stdint.h>
#include<math.h>
#include"pluginapi.h"

/* This plugin renders copperbars with amd64 inline asm. */

int w, h;
#define BT 15
const int barThickness = BT;
const int barLag = 5;

unsigned char grayBar[BT*4];
unsigned char redBar[BT*4];
unsigned char greenBar[BT*4];
unsigned char blueBar[BT*4];
unsigned char yellowBar[BT*4];
unsigned char purpleBar[BT*4];
unsigned char cyanBar[BT*4];


void buildBars();

void init(int width, int height, const char *config_file) {
    w = width;
    h = height;
    buildBars();
}

void buildBars() {
    int j;
    for (j=0; j<barThickness; j++) {
        unsigned char val = (unsigned char)(255.0*sin((double)j/barThickness*M_PI));
        grayBar[j*4] = val;
        grayBar[j*4 + 1] = val;
        grayBar[j*4 + 2] = val;
        grayBar[j*4 + 3] = 255;

        redBar[j*4] = val;
        redBar[j*4 + 1] = 0;
        redBar[j*4 + 2] = 0;
        redBar[j*4 + 3] = 255;

        greenBar[j*4] = 0;
        greenBar[j*4 + 1] = val;
        greenBar[j*4 + 2] = 0;
        greenBar[j*4 + 3] = 255;

        blueBar[j*4] = 0;
        blueBar[j*4 + 1] = 0;
        blueBar[j*4 + 2] = val;
        blueBar[j*4 + 3] = 255;

        yellowBar[j*4] = val;
        yellowBar[j*4 + 1] = val;
        yellowBar[j*4 + 2] = 0;
        yellowBar[j*4 + 3] = 255;

        purpleBar[j*4] = val;
        purpleBar[j*4 + 1] = 0;
        purpleBar[j*4 + 2] = val;
        purpleBar[j*4 + 3] = 255;

        cyanBar[j*4] = 0;
        cyanBar[j*4 + 1] = val;
        cyanBar[j*4 + 2] = val;
        cyanBar[j*4 + 3] = 255;
    }
}

void spread(unsigned char *texture) {
    int32_t *t = (int32_t*) texture;
    int i, j;
    for (j=0; j<h; j++) {
        int32_t value = t[j*w];
        for (i=1; i<w; i++) {
            t[j*w + i] = value;
        }
    }
}

void clearLeft(unsigned char *texture) {
    int j;
    for (j=0; j<h; j++) {
        asm("movb $0, (%0)\n\t"
            "movb $0, 1(%0)\n\t"
            "movb $0, 2(%0)\n\t"
            "movb $255, 3(%0)\n\t"
            :
            : "r" (texture + j*w*4)
        );
    }

}

void drawBar(unsigned char *texture, unsigned char *bar, int barTop) {
    uint32_t *dst = (uint32_t*) (texture + 4*w*barTop);
    uint32_t *src = (uint32_t*)bar;
    asm("shlq $2, %3\n\t"
        "CLOOP:\n\t"
        "movl (%1), %%edx\n\t"
        "movl %%edx, (%2)\n\t"
        "addq $4, %1\n\t"
        "addq %3, %2\n\t"
        "loop CLOOP\n\t"
        :
        : "c" ((int64_t) barThickness), "r" (src), "r" (dst), "r" ((uint64_t)w)
        : "edx"
        );
}

int calculateBarTop(long t) {
    return (int) (0.5*(h-barThickness) * (-sin(t/60.0*M_PI)+1.0));
}

void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    long tick = time_in_milliseconds / 17;
    clearLeft(texture);
    int barTop;

    barTop = calculateBarTop(tick);
    drawBar(texture, grayBar, barTop);
    barTop = calculateBarTop(tick - barLag);
    drawBar(texture, redBar, barTop);
    barTop = calculateBarTop(tick - 2*barLag);
    drawBar(texture, yellowBar, barTop);
    barTop = calculateBarTop(tick - 3*barLag);
    drawBar(texture, greenBar, barTop);
    barTop = calculateBarTop(tick - 4*barLag);
    drawBar(texture, cyanBar, barTop);
    barTop = calculateBarTop(tick - 5*barLag);
    drawBar(texture, blueBar, barTop);
    barTop = calculateBarTop(tick - 6*barLag);
    drawBar(texture, purpleBar, barTop);
    barTop = calculateBarTop(tick - 7*barLag);
    drawBar(texture, grayBar, barTop);

    if (w>1)
        spread(texture);
}

void gc(void) {
}

void shutdown(void) {
}
