/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <libcob.h>
#include "pluginapi.h"

extern int cobplug(double *x, double *z, double *time, double *sx, double *sy);
int w, h;

void init(int width, int height, const char *config_file) {
    cob_init(0, NULL);
    w = width;
    h = height;
}

void clearTexture(unsigned char *tex) {
    int i, j;

    for (j=0; j<h; j++) {
        for (i=0; i<w; i++) {
            unsigned char *base = tex + 4*(j*w + i);
            base[0] = 0;
            base[1] = 0;
            base[2] = 0;
        }
    }
}

void plot(unsigned char *tex, int x, int y, unsigned char r, unsigned char g, unsigned char b) {
    unsigned char *base;
    if (x <0 || x >= w || y < 0 || y >= h)
        return;
    base = tex + 4*(y*w + x);
    base[0] = r;
    base[1] = g;
    base[2] = b;
}

void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    double i, j, sx, sy;
    double time = time_in_milliseconds / 1000.0;
    const int si = 18;
    const int sj = 11;

    clearTexture(texture);
    for (i = 0; i <= sj; i++) {
        for (j = 0; j <= si; j++) {
            double tx, tz;
            unsigned char r, g, b;
            tx = ((double) j) / sj;
            tz = ((double) i) / si;
            if ((i >= 4 && i < 8) ||
                    (j >= 5 && j < 9)) {
                r = 24;
                g = 68;
                b = 126;
            } else {
                r = g = b = 255;
            }
            cobplug(&tx, &tz, &time, &sx, &sy);
            plot(texture, sx*150 + w/2, sy*150 + h/2, r, g, b);
        }
    }
}

void gc(void) {
}

void shutdown(void) {
}
