/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include<stdlib.h>
#include<math.h>
#include"pluginapi.h"

/* Implement a Mandelbrot zoomer in Fortran. */

#define TS 256

/* The Fortran evaluator function. */
int mandel_(int [][TS], int *texwidth, int *texheight,
        double *x0, double *y0, double *x1, double *y1,
        int *maxiter, double *cutoff);

int w, h;
int *iterations;

void init(int width, int height, const char *config_file) {
    w = width;
    h = height;
    iterations = malloc(w*h*sizeof(int));
}

void iterationsToTexture(int *iters, unsigned char *texture) {
    int i, j;
    for (i=0; i<w; i++) {
        for (j=0; j<h; j++) {
            unsigned char *base = texture + (j*w + i)*4;
            unsigned char val = (unsigned char) iters[j*w + i]*4;
            base[0] = val;
            base[1] = val;
            base[2] = val;
        }
    }
}


void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    double x0, x1, y0, y1, xc, yc, cutoff;
    double viewSize;
    int maxiter = 255;
    const int cycleInMillis = 10000;
    double time = (time_in_milliseconds % cycleInMillis)/((double) cycleInMillis);
    const double viewsizeMin = 0.0001;
    const double viewsizeMax = 0.11;
    double coordWeight;
    maxiter = 64;
    cutoff = 8.0;
    viewSize = viewsizeMin + (viewsizeMax-viewsizeMin)*(cos(time*4.0*M_PI) + 1);
    coordWeight = 0.5*(sin(time*2.0*M_PI) + 1);
    xc = coordWeight*0.0 + (1.0-coordWeight)*-0.158;
    yc = coordWeight*-1.0 + (1.0-coordWeight)*-1.113;
    x0 = xc - viewSize;
    y0 = yc - viewSize;
    x1 = xc + viewSize;
    y1 = yc + viewSize;
    mandel_((int (*)[TS])iterations, &w, &h, &x0, &y0, &x1, &y1, &maxiter, &cutoff);
    iterationsToTexture(iterations, texture);
}

void gc(void) {
}

void shutdown(void) {
    free(iterations);
    iterations=NULL;
}
