/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#import <math.h>
#import <stdint.h>
#import "Roto.h"

/* Not a member function due to efficiency. Assumes src and dst have
the same dimensions. */

void copyPixel(int x, int y, unsigned char *dst, int i,  int j, unsigned char *src, int w) {
  uint32_t *s;
  uint32_t *d;

  s = (uint32_t*)(src + 4*(y*w + x));
  d = (uint32_t*)(dst + 4*(j*w + i));
  *d = *s;
}

@implementation Roto

+ new {
  self = [super new];
  return self;
}

- free {
  return [super free];
}

- print {
  printf ("Roto object.\n");
  return self;
}

- (Roto*) setW: (int) width andH:(int) height andP:(unsigned char *) pixels {
  src = pixels;
  w = width;
  h = height;
  /* Assumes a power of two texture. */
  wMask = w-1;
  hMask = h-1;

  srcXOff = w/2;
  srcYOff = h/2;
  dstXOff = w/2;
  dstYOff = h/2;
  return self;
}

- (Roto*) setSrcXOff: (int) sxo andYOff: (int) syo {
  srcXOff = sxo;
  srcYOff = syo;
}

- (Roto*) setDstXOff: (int) dxo andYOff: (int) dyo {
  dstXOff = dxo;
  dstYOff = dyo;
}

- (Roto*) rotate: (double) angle andZoom: (double) zoom andDst: (unsigned char*) dst {
  int i, j;
  const double cost = cos(angle);
  const double sint = sin(angle);

  for (j=0; j<h; j++) {
    int curJ = zoom*(j - srcYOff);
    for (i=0; i<w; i++) {
      int curI = zoom*(i - srcXOff);
      int x = cost*curI - sint*curJ + dstXOff;
      int y = sint*curI + cost*curJ + dstYOff;
      x &= wMask;
      y &= hMask;
      
      copyPixel(x, y, dst, i, j, src, w);
    }
  }      
  return self;
}

@end
