/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* File modwrap.c -- wrappers around the Caml functions */

#include "pluginapi.h"
#include <stdio.h>
#include <string.h>
#include <caml/mlvalues.h>
#include <caml/callback.h>
#include <caml/bigarray.h>

struct {
    long width;
    long height;
    long size;
} dim;

void init(int width, int height, const char *config_file) {
    /* Initialize Caml runtime */
    char *argv[] = {strdup("altdemo"), NULL};
    caml_startup(argv);
    free(argv[0]);

    value *init_closure = caml_named_value("init");
    caml_callback2(*init_closure, Val_int(width), Val_int(height));

    dim.width = width;
    dim.height = height;
    dim.size = width * height;
}

/* Render a frame to the given preallocated texture buffer. Format
   is 32 bpp RGBA, size defined by init(). The time given should be
   used in preference to measuring time yourself. */
void render(unsigned char *texture, uint32_t time_ms) {
    static value *render_closure = NULL;
    if (render_closure == NULL) render_closure = caml_named_value("render");

    value textureVal = 
        caml_ba_alloc_dims(BIGARRAY_UINT8 | BIGARRAY_C_LAYOUT,
                           3, texture, dim.height, dim.width, 4);

    caml_callback2(*render_closure, textureVal, Val_int(time_ms));

    int i;
    for (i = 0; i < dim.size; i++)
        ((uint32_t *)texture)[i] |= 0xFF000000;
}

void gc(void) {
    /* Caml GC is a complex beast. Do not attempt to tame it if not needed. */
}

void shutdown(void) {
    /* Nothing to do here. */
}

int fib(int n)
{
    static value * fib_closure = NULL;
    if (fib_closure == NULL) fib_closure = caml_named_value("fib");
    return Int_val(caml_callback(*fib_closure, Val_int(n)));
}

char * format_result(int n)
{
    static value * format_result_closure = NULL;
    if (format_result_closure == NULL)
        format_result_closure = caml_named_value("format_result");
    return strdup(String_val(caml_callback(*format_result_closure, Val_int(n))));
    /* We copy the C string returned by String_val to the C heap
     so that it remains valid after garbage collection. */
}
