/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

int main(int argc, char **argv) {
    if (argc < 2) {
        fprintf(stderr, "Usage: %s <plugin.so> <config file>\n", argv[0]);
        exit(1);
    }
    char *plugin_name = argv[1];
    char *config_file = NULL;
    if (argc == 3) {
        config_file = argv[2];
        printf("Using %s as config file for plugin.\n", config_file);
    }

    printf("Loading plugin %s...\n", plugin_name);
    void *plugin_handle = dlopen(plugin_name, RTLD_NOW);
    if (!plugin_handle) {
        fprintf(stderr, "Error loading plugin: %s", dlerror());
        exit(1);
    }

    printf("Looking up init function...\n");
    void (*init_function)(int w, int h, const char *config_file);
    init_function = dlsym(plugin_handle, "init");
    if (!init_function) {
        fprintf(stderr, "Error looking up init function: %s", dlerror());
        exit(1);
    }

    printf("Looking up render function...\n");
    void (*render_function)(char *texture, int64_t time_in_micros);
    render_function = dlsym(plugin_handle, "render");
    if (!render_function) {
        fprintf(stderr, "Error looking up render function: %s", dlerror());
        exit(1);
    }

    printf("Calling init function...\n");
    init_function(320, 240, config_file);

    printf("Calling render function...\n");
    render_function(NULL, 0);

    printf("Closing plugin...\n");
    dlclose(plugin_handle);
    return 0;
}
