/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <assert.h>
#include "SDL.h"
#include "SDL_image.h"
#include "pluginapi.h"
#include "ruby.h"

SDL_Surface *backImage;

#define MAXSIZE 1200

char scriptHolder[MAXSIZE] = {0};
int time_in_millis;
unsigned char *tex;

static VALUE copyline(VALUE obj, VALUE arg1, VALUE arg2) {
    int sourceLineNum = NUM2INT(arg1);
    int destLineNum = NUM2INT(arg2);
    memcpy(tex + destLineNum * backImage->pitch, backImage->pixels
            + sourceLineNum * backImage->pitch, backImage->pitch);
    return Qnil;
}

static VALUE gettime(VALUE obj)  {
    return INT2NUM(time_in_millis);
}

static VALUE getheight(VALUE obj)  {
    return INT2NUM(backImage->h);
}

void init_ruby(const char *script) {
    FILE *f;
    f = fopen(script, "r");
    fread(scriptHolder, MAXSIZE, 1, f);
    fclose(f);
    ruby_init();
    rb_define_global_function("copyline", (VALUE(*)(ANYARGS)) copyline, 2);
    rb_define_global_function("gettime", (VALUE(*)(ANYARGS)) gettime, 0);
    rb_define_global_function("getheight", (VALUE(*)(ANYARGS)) getheight, 0);
    ruby_script("embed");
    rb_load_file(script);
}

void init(int width, int height, const char *config_file) {
    backImage = IMG_Load("../data/matsi.png");
    if (!backImage) {
        printf("Image load is fail\n");
        return;
    }
    assert(backImage->pitch == 4*backImage->w);
    assert(backImage->w == width);
    assert(backImage->h == height);
    init_ruby(config_file);
}

void copyLine(unsigned char *texture, const int sourceLineNum,
        const int destLineNum) {
    memcpy(texture + destLineNum * backImage->pitch, backImage->pixels
            + sourceLineNum * backImage->pitch, backImage->pitch);
}

void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    time_in_millis = time_in_milliseconds;
    tex = texture;
    rb_eval_string(scriptHolder);
}

void gc(void) {
    rb_eval_string("GC.start()");
}

void shutdown(void) {
    SDL_FreeSurface(backImage);
    ruby_finalize();
}
