/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <assert.h>
#include "SDL.h"
#include "pluginapi.h"

/* Spawn shell script to render interference pattern. */

#define BUFSIZE 1024
char scriptFile[BUFSIZE];

void init(int width, int height, const char *config_file) {
    assert(width == 256);
    assert(height == 256);
    strncpy(scriptFile, config_file, BUFSIZE);
}

void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    char cmd[1024];
    SDL_Surface *loader;
    sprintf(cmd, "%s %d", scriptFile, time_in_milliseconds);
    system(cmd);
    loader = SDL_LoadBMP("shell-out.bmp");
    assert(loader->pitch == loader->w*loader->format->BitsPerPixel/8);
    if (loader->format->BitsPerPixel == 32) {
        memcpy(texture, loader->pixels, loader->h*loader->pitch);
    } else if (loader->format->BitsPerPixel == 24) {
        unsigned char *src = loader->pixels;
        unsigned char *dst = texture;
        int i;
        for (i=0; i<loader->w*loader->h; i++) {
                dst[0] = src[0];
                dst[1] = src[1];
                dst[2] = src[2];
                dst += 4;
                src += 3;
        }

    } else {
        printf("Image loading FAIL.\n");
        exit(1);
    }
    SDL_FreeSurface(loader);
}

void gc(void) {
}

void shutdown(void) {
}
