/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <SDL.h>
#include <assert.h>
#include <mono/jit/jit.h>
#include <mono/metadata/mono-config.h>
#include <mono/metadata/assembly.h>
#include <mono/metadata/debug-helpers.h>

#include"pluginapi.h"

char *empty; /* This is only needed to call Mono's internals. It is not used. */

MonoDomain *domain;
MonoAssembly *assembly;
MonoImage *image;
MonoClass *demoClass;
MonoObject *demoInstance;
MonoMethodDesc* evalMethodDesc;
MonoMethod* evalMethod;

void init(int width, int height, const char *config_file) {
    empty = malloc(2);
    empty[0] = 'X';
    empty[1] = '\0';

    mono_config_parse(NULL);
    domain = mono_jit_init(config_file);

    assembly = mono_domain_assembly_open(domain, config_file);
    if (!assembly) {
        printf("Failed to open assembly %s.\n", config_file);
        error ();
    }
    image = mono_assembly_get_image(assembly);
    demoClass = mono_class_from_name(image, "", "Effect");
    demoInstance = mono_object_new(domain, demoClass);
    evalMethodDesc = mono_method_desc_new("Effect:render(int)", FALSE);
    evalMethod = mono_method_desc_search_in_class(evalMethodDesc, demoClass);

    mono_runtime_object_init(demoInstance);
}

/* Because accessing Mono's internal object data
 * is hard, we have this horrible hack.
 */
void readOutput(unsigned char *tex) {
    int i, j;
    SDL_Surface *loader = SDL_LoadBMP("vb-out.bmp");
    assert(loader->format->BitsPerPixel == 32);
    /* Mono writes color channels in a different order, so swap them. */
    for (j=0; j<loader->h; j++) {
        unsigned char *src = loader->pixels + j*loader->pitch;
        unsigned char *dst = tex + j*loader->pitch;
        for (i=0; i<loader->w; i++) {
            dst[0] = src[2];
            dst[1] = src[1];
            dst[2] = src[0];
            dst += 4;
            src += 4;
        }
    }
    SDL_FreeSurface(loader);
}

void render(unsigned char *texture, uint32_t time_in_milliseconds) {
    void *args[1];
    int t = time_in_milliseconds;
    args[0] = &t;
    mono_runtime_invoke(evalMethod, demoInstance, args, NULL);
    readOutput(texture);
}

void gc(void) {

}

void shutdown(void) {
    mono_method_desc_free(evalMethodDesc);
    mono_jit_cleanup(domain);
    free(empty);
}
