/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include"SDL.h"
#include"SDL_opengl.h"
#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#include"demoplugin.h"
#include"pluginthread.h"
#include"config.h"

const int WIDTH = 800;
const int HEIGHT = 600;
const int TEXSIZE = 256;

SDL_Surface *init_graphics(int w, int h, bool fullscreen) {
    Uint32 surfaceFlags = SDL_OPENGL;
    SDL_Surface *s;
    int depth = 32;

    if(fullscreen)
        surfaceFlags |= SDL_FULLSCREEN;

    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    s = SDL_SetVideoMode(w, h, depth, surfaceFlags);
    return s;
}

void opengl_setup() {
    glShadeModel(GL_SMOOTH);
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(45.0f,(GLfloat)WIDTH/(GLfloat)HEIGHT,0.1f,100.0f);

    glMatrixMode(GL_MODELVIEW);

}

void simple_draw(double dx, double dy) {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    glTranslated(dx, dy, -6.0f);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
    glEnd();

    glFinish();
}

void viewTest(const char *pluginfile, const char *conffile) {
    SDL_Event e;
    Uint32 firstFrameTime;
    PluginThread plt(pluginfile, TEXSIZE, TEXSIZE, conffile);

    glColor3d(1.0, 1.0, 1.0);
    GLuint texid[1];
    Uint32 fpsCountStart;
    int framesRendered = 0;
    int mainIterated = 0;

    glGenTextures(1, &texid[0]);
    glBindTexture(GL_TEXTURE_2D, texid[0]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

    firstFrameTime = fpsCountStart =  SDL_GetTicks();
    plt.continueToNextFrame(0);

    while(1) {
        const Uint32 frameTimeWith60FPS = 17;
        Uint32 startTime = SDL_GetTicks();
        Uint32 endTime, elapsed;
        if (plt.hasNewFrame()) {
            glTexImage2D(GL_TEXTURE_2D, 0, 4, TEXSIZE, TEXSIZE, 0, GL_RGBA, 
                         GL_UNSIGNED_BYTE, plt.getTexture());
            simple_draw(0.0, 0.0);
            plt.continueToNextFrame(startTime - firstFrameTime);
            SDL_GL_SwapBuffers();
            framesRendered++;
        }
        
        // Loop at 60 FPS.
        endTime = SDL_GetTicks();
        elapsed = endTime - startTime;
        if (elapsed < frameTimeWith60FPS) {
            SDL_Delay(frameTimeWith60FPS - elapsed);
        }
        mainIterated++;

        // Print FPS rate.
        if (SDL_GetTicks() - fpsCountStart > 1000) {
            printf("FPS: %d - main iterations: %d\n", framesRendered, mainIterated);
            framesRendered = 0;
            mainIterated = 0;
            fpsCountStart = SDL_GetTicks();
        }

        // Check if we need to exit.
        while(SDL_PollEvent(&e)) {
            if (e.type == SDL_QUIT ||
                (e.type == SDL_KEYDOWN &&
                (e.key.keysym.sym == SDLK_ESCAPE || e.key.keysym.sym == SDLK_q))) {
                glDeleteTextures(1, &texid[0]);
                plt.shutdown();
                return;
            }
        }
    }
}

int main(int argc, char **argv) {
    const int isFullscreen = false;
    SDL_Surface *canvas;
    const char *conffile;

    if(argc < 2) {
        printf("No plugin specified.\n");
        return 1;
    }
    if(strstr(argv[1], "/") == 0) {
        printf("Slash missing from plugin file name. Exiting as this will most likely fail.\n");
        return 1;
    }

    printf("Simple plugin viewer\n");

    if(SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0) {
        fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        return 1;
    }
    SDL_WM_SetCaption("Plugin viewer", "Plugin viewer");

    canvas = init_graphics(WIDTH, HEIGHT, isFullscreen);
    if(!canvas) {
        fprintf(stderr, "Could not initialize video: %s\n", SDL_GetError());
        exit(1);
    }

    if(argc<2)
        conffile = 0;
    else
        conffile = argv[2];
    opengl_setup();
    viewTest(argv[1], conffile);

    SDL_Quit();
    return 0;
}
