/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PLUGINTHREAD_H
#define PLUGINTHREAD_H

#include <boost/thread.hpp>
#include <inttypes.h>

#include "demoplugin.h"

class PluginCallable;

class PluginThread {
public:
    /* Constructs a plugin hosting thread. Constructor arguments are passed to the plugin. */
    PluginThread(const char *pluginName, int width, int height, const char *configFile);

    /* True if a new frame has been rendered since last getTexture() call */
    bool hasNewFrame(void);

    /* Returns the texture from plugin. If the calculation is running, this
       will wait until the frame is completed. */
    unsigned char* getTexture();

    /* Tells the thread to start calculating */
    void continueToNextFrame(uint32_t time_milliseconds);
    
    /* Stop the thread and try to clean up after the plugin */
    void shutdown(void);

    ~PluginThread();

    /* TODO: should the main thread do the GC calls on plugin thread? */

    int getTextureWidth() const { return plugin.getTextureWidth(); }
    int getTextureHeight() const { return plugin.getTextureHeight(); }

private:
    DemoPlugin plugin;
    boost::thread thread;
    boost::mutex mutex;
    boost::condition_variable cond;
    PluginCallable *callable;

    uint32_t prevFrameTime;
    uint32_t nextFrameTime;
    bool newFrame;
};

#endif
