/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SCENERUNNER_H
#define SCENERUNNER_H

#include <vector>
#include <string>
#include "SDL_opengl.h"
#include "pluginthread.h"
#include "videomanager.h"

class DemoRunner;
struct basicScene;

enum sceneType {
    testScene,
    cubeScene,
    emptyScene,
    isectScene
};

/*
 * This class holds all the code to run the different scene types.
 *
 * It is very closely linked to DemoRunner.
 */

class SceneRunner {
private:

    DemoRunner &d;
    VideoManager &v;
    Uint32 sceneStartTime;
    int framesRenderedInThisScene;
    void checkForExit();
    void getTexStuff(std::vector<std::string> &plugins, std::vector<PluginThread*> &threads, std::vector<GLuint> &texids);
    void updateTextures(Uint32 frameStart, std::vector<PluginThread*> &threads, std::vector<GLuint> &texids);
    void delayForFrameFlip(int numFramesRendered, Uint32 startTime, Uint32 frameStart, const char *sceneName);
    void runIt(basicScene &s, sceneType type);

    // These functions draw the actual scenes
    void renderTestScene(std::vector<GLuint> &texids, Uint32 timeSinceSceneStart);
    void renderCubeScene(std::vector<GLuint> &texids, Uint32 timeSinceSceneStart);
    void renderIntersectScene(std::vector<GLuint> &texids, Uint32 timeSinceSceneStart);

public:

    SceneRunner(DemoRunner &dr, VideoManager &vm);
    void slideshowLoop();
    void runCubeScene(Uint32 startTime, basicScene &s);
    void runIntersectScene(Uint32 startTime, basicScene &s);
    void runTestScene(Uint32 startTime, basicScene &s);
    void runEmptyScene(Uint32 startTime, basicScene &s);
    void runRayScene(Uint32 startTime, basicScene &s);

};

#endif
