/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include"SDL.h"
#include"SDL_opengl.h"
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<vector>

#include"demoplugin.h"
#include"config.h"

using namespace std;

struct gltex {
    DemoPlugin *pl;
    GLuint texid[1];
};

/* A test program to determine how many plungins can transfer their
 * textures to screen on every frame.
 */

const int WIDTH = 800;
const int HEIGHT = 600;
const int TEXRES = 256;
const int NUMFRAMES = 720;

SDL_Surface *init_graphics(int w, int h, bool fullscreen) {
    Uint32 surfaceFlags = SDL_OPENGL;
    SDL_Surface *s;
    int depth = 32;

    if(fullscreen)
        surfaceFlags |= SDL_FULLSCREEN;

    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    s = SDL_SetVideoMode(w, h, depth, surfaceFlags);
    return s;
}

void load_plugins(vector<gltex> &arr, const int n) {
    const char *pluginname = "./libdummy.so";
    for(int i=0; i<n; i++) {
        gltex t;
        t.pl = new DemoPlugin(pluginname);
        t.pl->load();
        t.pl->init(TEXRES, TEXRES, 0);
        glGenTextures(1, &t.texid[0]);
        glBindTexture(GL_TEXTURE_2D, t.texid[0]);

        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

        t.pl->render(0);
        glTexImage2D(GL_TEXTURE_2D, 0, 4, TEXRES, TEXRES, 0, GL_RGBA, GL_UNSIGNED_BYTE,
                t.pl->getTexture());
        arr.push_back(t);

    }
}

void opengl_setup() {
    glShadeModel(GL_SMOOTH);
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    gluPerspective(45.0f,(GLfloat)WIDTH/(GLfloat)HEIGHT,0.1f,100.0f);

    glMatrixMode(GL_MODELVIEW);

}

void simple_draw(double dx, double dy) {
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslated(dx, dy, -6.0f);
    glBegin(GL_QUADS);
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
    glEnd();

}

void transferTest(int numPlugins, Uint32 &transferTime, Uint32 &genTime, Uint32 &drawTime) {
    SDL_Event e;
    vector<gltex> plugins;
    load_plugins(plugins, numPlugins);
    glColor3d(1.0, 1.0, 1.0);
    transferTime = genTime = drawTime = 0;

    for(double i=0.0; i<NUMFRAMES; i++) {
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        for(int numTex=0; numTex<numPlugins; numTex++) {
            Uint32 startTime, endTime;
            double angleOffset = 360.0/numPlugins*numTex;
            double dx = 1.5*cos((i+angleOffset)/360.0*2*M_PI);
            double dy = 1.5*sin((i+angleOffset)/360.0*2*M_PI);
            gltex curtex = plugins[numTex];

            startTime = SDL_GetTicks();
            curtex.pl->render(0);
            endTime = SDL_GetTicks();
            genTime += endTime-startTime;

            startTime = SDL_GetTicks();
            glBindTexture(GL_TEXTURE_2D, curtex.texid[0]);
            startTime = SDL_GetTicks();
            glTexImage2D(GL_TEXTURE_2D, 0, 4, TEXRES, TEXRES, 0, GL_RGBA, GL_UNSIGNED_BYTE,
                    curtex.pl->getTexture());
            endTime = SDL_GetTicks();
            transferTime += endTime-startTime;

            startTime = SDL_GetTicks();
            simple_draw(dx, dy);
            endTime = SDL_GetTicks();
            drawTime += endTime-startTime;

        }
        glFinish();
        SDL_GL_SwapBuffers();
        // Check if we need to exit prematurely.
        while(SDL_PollEvent(&e)) {
            if (e.type == SDL_QUIT ||
                (e.type == SDL_KEYDOWN && e.key.keysym.sym == SDLK_ESCAPE))
                return;
        }
    }
    return;
}

int main(int argc, char **argv) {
    const int isFullscreen = false;
    SDL_Surface *canvas;
    Uint32 startTime, endTime;
    Uint32 totalTime, transferTime, genTime, drawTime;
    int numPlugins;

    try {
        if(SDL_Init(SDL_INIT_AUDIO|SDL_INIT_VIDEO) < 0) {
            fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
            return 1;
        }
        SDL_WM_SetCaption("Transfer speed test", "Transfer speed test");

        canvas = init_graphics(WIDTH, HEIGHT, isFullscreen);
        if(!canvas) {
            fprintf(stderr, "Could not initialize video: %s\n", SDL_GetError());
            exit(1);
        }

        if(argc < 2)
            numPlugins = 4;
        else
            numPlugins = atoi(argv[1]);

        if(numPlugins < 1) {
            printf("You iz fail!\n");
            return 1;
        }

        opengl_setup();

        printf("Using %d plugins.\n", numPlugins);
        printf("Texture size %d.\n", TEXRES);
        printf("Screen resolution %dx%d.\n", WIDTH, HEIGHT);

        startTime = SDL_GetTicks();
        transferTest(numPlugins, transferTime, genTime, drawTime);
        endTime = SDL_GetTicks();

        totalTime = endTime - startTime;
        printf("FPS: %.2f\n", 1000.0*NUMFRAMES/totalTime);
        printf("Percentage of time spent uploading textures:  %.2f\n", double(transferTime)/totalTime);
        printf("Percentage of time spent generating textures: %.2f\n", double(genTime)/totalTime);
        printf("Percentage of time spent drawing textures:    %.2f\n", double(drawTime)/totalTime);
        SDL_Quit();
    } catch (char *msg) {
        fprintf(stderr, "Exception: %s", msg);
        return 1;
    }
    return 0;
}
