/*
Binary of Babel
Copyright (C) 2010 Segfault Garden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef VIDEOMANAGER_H
#define VIDEOMANAGER_H

#include "SDL_opengl.h"
#include "SDL.h"
#include <vector>

class VideoManager {

private:
    int screenWidth, screenHeight;
    bool isFullscreen;

    void initOpenGL();
    void drawBasicPlane();

public:

    VideoManager(int width, int height, bool fullScr);
    void initGraphics();
    GLuint createTexture(int w, int h, int depth, bool smooth, GLuint pixelFormat, unsigned char *pixels);
    void updateTexture(GLuint texid, int width, int height, unsigned char *pixels);
    void switchTo2DMode();
    void switchTo3DMode();
    void drawOverlay(GLuint tex, double x, double y, double width, double height, double opacity);
    void simpleDraw(double dx, double dy);
    void drawCube(GLuint tex);
    void drawCube(std::vector<GLuint> &texes);
    void bind(GLuint texid) { glBindTexture(GL_TEXTURE_2D, texid); }
    void shutdown();
    void deleteTexture(GLuint tex) { glDeleteTextures(1, &tex); }
    void clear() { glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); }
    void flip() { SDL_GL_SwapBuffers(); }
};

#endif
